-- BuffCheck.lua
-- by Mithrandir1210
-- Last Updated: 11/27/2015

-- SETUP/LOADING --======================================================

local VersionNum = "1.0.3"

BuffCheck = {}; -- Global addon table

-- CREATE FRAMES --
local BuffCheck_EventFrame = CreateFrame("Frame")

-- POPULATE INTERFACE OPTIONS --===============================================

-- Add entry to interface options
BuffCheck.panel = CreateFrame ("Frame", "BuffCheck_Panel", UIParent);
BuffCheck.panel.name = "Buff Check";
InterfaceOptions_AddCategory(BuffCheck.panel);

-- Create title 
local BuffCheck_SettingsTitle = BuffCheck.panel:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
BuffCheck_SettingsTitle:SetPoint("TOPLEFT", 10, -15)
BuffCheck_SettingsTitle:SetText("Buff Check Options")

-- Create checkbuttons and their labels

-- Announce to raid?
local BuffCheck_btnAnnounce = CreateFrame ("CheckButton", nil, BuffCheck.panel, "InterfaceOptionsCheckButtonTemplate")
BuffCheck_btnAnnounce:SetPoint("TOPLEFT", 20, -40)
local BuffCheck_btnAnnounce_Text = BuffCheck_btnAnnounce:CreateFontString(nil, "ARTWORK", "GameFontHighlight")
BuffCheck_btnAnnounce_Text:SetPoint("LEFT", BuffCheck_btnAnnounce, "RIGHT", 0, -1)
BuffCheck_btnAnnounce_Text:SetText("Announce missing buffs to group")

-- Check after readycheck?
local BuffCheck_btnReadycheck = CreateFrame ("CheckButton", nil, BuffCheck.panel, "InterfaceOptionsCheckButtonTemplate")
BuffCheck_btnReadycheck:SetPoint("TOPLEFT", 20, -70)
local btnReadycheck_Text = BuffCheck_btnReadycheck:CreateFontString(nil, "ARTWORK", "GameFontHighlight")
btnReadycheck_Text:SetPoint("LEFT", BuffCheck_btnReadycheck, "RIGHT", 0, -1)
btnReadycheck_Text:SetText("Check for buffs after readycheck")

-- Announce as raid warning (only works if "announce to raid" is enabled)
-- <to be implemented>

-- REGISTER EVENTS --
BuffCheck_EventFrame:RegisterEvent("ADDON_LOADED")
BuffCheck_EventFrame:RegisterEvent("READY_CHECK_FINISHED")

-- SAVED VARIABLE TABLE
BuffCheck_SavedVars = {}

-- COLOR PREFIXES --
local BuffCheck_Colors = {}
BuffCheck_Colors.Red = "|cFFFF0000"
BuffCheck_Colors.Blue = "|cFF0000FF"
BuffCheck_Colors.Green = "|cFF008000"
BuffCheck_Colors.Yellow = "|cFFFFFF00"
BuffCheck_Colors.Brown = "|cFF663300"
BuffCheck_Colors.Gray = "|cFF808080"
BuffCheck_Colors.White = "|cFFFFFFFF"
BuffCheck_Colors.Black = "|cFF000000"
BuffCheck_Colors.Aquablue = "|cFF00B8E6"

-- DEFAULT SETTINGS --
local BuffCheck_Defaults = {}
BuffCheck_Defaults.FontColor = BuffCheck_Colors.Aquablue
BuffCheck_Defaults.AnnounceMissing = false
BuffCheck_Defaults.CheckAtReadycheck = false

-- OTHER LOCAL VARIABLES --
local NumPlayers

-- All Possible Raid Buffs
local StatsBuffs = {"Mark of the Wild",
					"Legacy of the Emperor",
					"Legacy of the White Tiger",
					"Blessing of Kings",
					"Lone Wolf: Power of the Primates",
					"Bark of the Wild",
					"Blessing of Kongs",
					"Embrace of the Shale Spider",
					"Strength of the Earth"}
local StamBuffs = {"Power Word: Fortitude",
				   "Blood Pact",
				   "Commanding Shout",
				   "Lone Wolf: Fortitude of the Bear",
				   "Invigorating Roar",
				   "Sturdiness",
				   "Savage Vigor",
				   "Qiraji Fortitude"}
				   
local ApBuffs = {"Trueshot Aura",
			     "Battle Shout",
				 "Horn of Winter"}
local SpBuffs = {"Arcane Brilliance",
				 "Dalaran Brilliance",
				 "Dark Intent",
				 "Lone Wolf: Wisdom of the Serpent",
				 "Serpent's Cunning",
				 "Qiraji Fortitude",
				 "Still Water"}
local MastBuffs = {"Grace of Air",
		       	   "Moonkin Aura",
				   "Blessing of Might",
				   "Power of the Grave",
				   "Lone Wolf: Grace of the Cat",
				   "Roar of Courage",
				   "Keen Senses",
				   "Spirit Beast Blessing",
				   "Plainswalking"}
local HasteBuffs = {"Unholy Aura",
					"Mind Quickening",
					"Swiftblade's Cunning",
					"Grace of Air",
					"Lone Wolf: Haste of the Hyena",
					"Cackling Howl",
					"Savage Vigor",
					"Energizing Spores",
					"Speed of the Swarm"}
local CritBuffs = {"Leader of the Pack",
				   "Arcane Brilliance",
				   "Dalaran Brilliance",
				   "Legacy of the White Tiger",
				   "Lone Wolf: Ferocity of the Raptor",
				   "Terrifying Roar",
				   "Fearless Roar",
				   "Strength of the Pack",
				   "Embrace of the Shale Spider",
				   "Still Water",
				   "Furious Howl"}
local MultiBuffs = {"Dark Intent",
					"Windflurry",
					"Mind Quickening",
					"Swiftblade's Cunning",
					"Lone Wolf: Quickness of the Dragonhawk",
					"Sonic Focus",
					"Wild Strength",
					"Double Bite",
					"Spry Attacks",
					"Breath of the Winds"}
local VersBuffs = {"Mark of the Wild",
				   "Unholy Aura",
				   "Sanctity Aura",
				   "Inspiring Presence",
				   "Lone Wolf: Versatility of the Ravager",
				   "Tenacity",
				   "Indomitable",
				   "Wild Strength",
				   "Defensive Quills",
				   "Chitinous Armor",
				   "Grace",
				   "Strength of the Earth"}

local GreaterFlasks = {"Greater Draenic Agility Flask",
				   "Greater Draenic Intellect Flask",
				   "Greater Draenic Strength Flask",
				   "Greater Draenic Stamina Flask"}
				   
local FoodBuff = {"Well Fed"}

local AugmentRune = {"Stout Augmentation",
					 "Focus Augmentation",
					 "Hyper Augmentation"}
					 
-- Other Buffs (not implemented yet)
local RegularFlasks = {"Draenic Agility Flask",
					   "Draenic Intellect Flask",
					   "Draenic Stamina Flask"}


-- Meta Table for buffs
local BuffMeta = {StatsBuffs, StamBuffs, ApBuffs, SpBuffs, MastBuffs,
                  HasteBuffs, CritBuffs, MultiBuffs, VersBuffs,
                  GreaterFlasks, FoodBuff, AugmentRune}

-- Players Missing Buffs
local MissingStats = {}
local MissingStam = {}
local MissingAp = {}
local MissingSp = {}
local MissingMast = {}
local MissingHaste = {}
local MissingCrit = {}
local MissingMulti = {}
local MissingVers = {}
local MissingFlask = {}
local MissingFood = {}
local MissingRune = {}

-- Other Missing Buffs (not implemented yet)
local MissingRegularFlask = {}

-- Meta table for missing buffs
local MissingMeta = {MissingStats, MissingStam, MissingAp, MissingSp,
                     MissingMast, MissingHaste, MissingCrit, MissingMulti,
                     MissingVers, MissingFlask, MissingFood, MissingRune}

-- LOCAL FUNCTIONS --
local BuffCheck_Display
local BuffCheck_SetChannel

local BuffCheck_ClearTables
local BuffCheck_IsBuffFound
local BuffCheck_PrintMissingBuffs
local BuffCheck_CheckBuffs

local BuffCheck_ViewSettings
local BuffCheck_SetDefaults

local BuffCheck_SlashCmdHandler

-- FUNCTIONS --================================================================

-------------------------------------------------------------------------------
-- Displays a message to chat or locally to the console if no ChatType or
--   ChatType == "console"
-- Param1: Message to display (string);
-- Param2 (optional): Send message in chat or console ("RAID", "SAY", "console", etc)
-- Param3 (optional): Color of text to display (only available for console)
-------------------------------------------------------------------------------
BuffCheck_Display = function(MessageString, ChatType, FontColor)
	-- Set color to default if no color is specified
	if not FontColor then
		FontColor = BuffCheck_Defaults.FontColor
	end
	
	-- If a chat type is specified, use SendChatMessage() instead of print()
	-- Display to chat
	if ChatType and ChatType ~= "console" then
	    MessageString = "[BC] " .. MessageString
        SendChatMessage(MessageString, ChatType, nil, nil)
	-- Display to console
    elseif not ChatType or ChatType == "console" then
	    MessageString = FontColor.."[BC] "..MessageString
	    print(MessageString)
    else 
		MessageString = FontColor.."[BC] "..MessageString
	    print(MessageString)
	end
    
end

-------------------------------------------------------------------------------
-- Sets all elements in all tables for missing buffs to nil.
-------------------------------------------------------------------------------
BuffCheck_ClearTables = function ()
    for i=1, #MissingMeta do
        for j=1, #MissingMeta[i] do
            MissingMeta[i][j] = nil
        end
    end
end

-------------------------------------------------------------------------------
-- Checks whether a type of buff is found on the unit
-------------------------------------------------------------------------------
BuffCheck_IsBuffFound = function (BuffTableName, TargetOfCheck)

	for i=1, #BuffTableName do
		if (select(1,UnitBuff(TargetOfCheck, BuffTableName[i]))) then
			return true
		end
	end
		
	return false
	
end -- IsBuffFound()

-------------------------------------------------------------------------------
-- Displays all players missing certain buffs; Prints to chat or console
-- Param1: Shows if all buffs are missing (boolean)
-------------------------------------------------------------------------------
BuffCheck_PrintMissingBuffs = function (IsMissing, GroupType)
	
	local ChannelType = GroupType
	
	-- Set and error-check channel type
	if ChannelType == "RAID" or ChannelType == "raid" or ChannelType == "Raid" then
		ChannelType = "RAID"
	elseif ChannelType == "PARTY" or ChannelType == "party" or ChannelType == "Party" then
		ChannelType = "PARTY"
	elseif ChannelType == "SOLO" or ChannelType == "solo" or ChannelType == "Solo" then
		ChannelType = "SOLO"
	else
		ChannelType = "console"
	end
	
	-- Display to "SAY" or console based on settings
	if not BuffCheck_SavedVars.AnnounceMissing then
		ChannelType = "console"
	elseif BuffCheck_SavedVars.AnnounceMissing and ChannelType == "SOLO" then
		ChannelType = "SAY"
	end
	    
	-- Display only if buffs are missing
	if IsMissing then
		BuffCheck_Display("Checking for missing buffs...",ChannelType)
	else
		BuffCheck_Display("All buffs found.",ChannelType)
	end
	
	-- Concatenate players missing buffs into table and display to proper channel
	if #MissingStats >= 1 then
		BuffCheck_Display("Missing Stats: "..table.concat(MissingStats,", "),ChannelType)
	end
	if #MissingStam >= 1 then
		BuffCheck_Display("Missing Stamina: "..table.concat(MissingStam,", "),ChannelType)
	end
	if #MissingAp >= 1 then
		BuffCheck_Display("Missing Attack Power: "..table.concat(MissingAp,", "),ChannelType)
	end
	if #MissingSp >= 1 then
		BuffCheck_Display("Missing Spell Power: "..table.concat(MissingSp,", "),ChannelType)
	end
	if #MissingMast >= 1 then
		BuffCheck_Display("Missing Mastery: "..table.concat(MissingMast,", "),ChannelType)
	end
	if #MissingCrit >= 1 then
		BuffCheck_Display("Missing Crit: "..table.concat(MissingCrit,", "),ChannelType)
	end
	if #MissingHaste >= 1 then
		BuffCheck_Display("Missing Haste: "..table.concat(MissingHaste,", "),ChannelType)
	end
	if #MissingMulti >= 1 then
		BuffCheck_Display("Missing Multistrike: "..table.concat(MissingMulti,", "),ChannelType)
	end
	if #MissingVers >= 1 then
		BuffCheck_Display("Missing Versatility: "..table.concat(MissingVers,", "),ChannelType)
	end
	if #MissingFood >= 1 then
		BuffCheck_Display("Missing Food: "..table.concat(MissingFood,", "),ChannelType)
	end
	if #MissingFlask >= 1 then
		BuffCheck_Display("Missing Flask: "..table.concat(MissingFlask,", "),ChannelType)
	end
	if #MissingRune >= 1 then
		BuffCheck_Display("Missing Rune: "..table.concat(MissingRune,", "),ChannelType)
	end
	
end -- PrintMissingBuffs()

-------------------------------------------------------------------------------
-- Checks for buffs and adds names of players missing buffs to a table.
-- Calls PrintMissingBuffs().
-------------------------------------------------------------------------------
BuffCheck_CheckBuffs = function()
    
    IsMissingBuff = false
    
	-- Raid group
    if IsInRaid() then
        NumPlayers = GetNumGroupMembers()
        
        for i=1, NumPlayers do
            
            for j=1, #BuffMeta do
                if not BuffCheck_IsBuffFound(BuffMeta[j],"raid"..i) then
                    table.insert(MissingMeta[j], (select(1,UnitName("raid"..i))))
                    IsMissingBuff = true
                end
            end
        end
    
        BuffCheck_PrintMissingBuffs(IsMissingBuff, "RAID")
        
        return -- to prevent double printing
    
	-- Party
	elseif not IsInRaid() and IsInGroup(LE_PARTY_CATEGORY_HOME) then
		NumPlayers = GetNumGroupMembers()
        
        for i=1, NumPlayers do
            
            for j=1, #BuffMeta do
                if not BuffCheck_IsBuffFound(BuffMeta[j],"party"..i) then
                    table.insert(MissingMeta[j], (select(1,UnitName("party"..i))))
                    IsMissingBuff = true
				end
            end
			
        end
		
		for k=1, #BuffMeta do
			if not BuffCheck_IsBuffFound(BuffMeta[k],"player") then
				table.insert(MissingMeta[k], (select(1,UnitName("player"))))
				IsMissingBuff = true
			end
        end
    
        BuffCheck_PrintMissingBuffs(IsMissingBuff, "PARTY")
        
        return
    
	-- Instance group
	elseif IsInGroup(LE_PARTY_CATEGORY_INSTANCE) then
		NumPlayers = GetNumGroupMembers()
        
        for i=1, NumPlayers do
            
            for j=1, #BuffMeta do
                if not BuffCheck_IsBuffFound(BuffMeta[j],"raid"..i) then
                    table.insert(MissingMeta[j], (select(1,UnitName("raid"..i))))
                    IsMissingBuff = true
                end
            end
        end
    
        BuffCheck_PrintMissingBuffs(IsMissingBuff, "RAID")
        
        return
	
	-- Solo or other
    else
        
        for i=1, #BuffMeta do
            if not BuffCheck_IsBuffFound(BuffMeta[i],"player") then
                table.insert(MissingMeta[i], (select(1,UnitName("player"))))
                IsMissingBuff = true
            end
        end
        
        BuffCheck_PrintMissingBuffs(IsMissingBuff, "SOLO")
    end -- if IsInRaid
    
end -- CheckBuffs()

-------------------------------------------------------------------------------
-- Print current settings to the console.
-------------------------------------------------------------------------------
BuffCheck_ViewSettings = function()

	BuffCheck_Display("Announce to raid?: "..tostring(BuffCheck_SavedVars.AnnounceMissing))
	BuffCheck_Display("Check buffs at readycheck?: "..tostring(BuffCheck_SavedVars.CheckAtReadycheck))

end

-------------------------------------------------------------------------------
-- Reset settings to their defaults.
-------------------------------------------------------------------------------
BuffCheck_SetDefaults = function()
	
	BuffCheck_ClearTables()
		
	BuffCheck_SavedVars.AnnounceMissing = BuffCheck_Defaults.AnnounceMissing
	BuffCheck_btnAnnounce:SetChecked(BuffCheck_Defaults.AnnounceMissing)
	
	BuffCheck_SavedVars.CheckAtReadycheck = BuffCheck_Defaults.CheckAtReadycheck
	BuffCheck_btnReadycheck:SetChecked(BuffCheck_Defaults.CheckAtReadycheck)
	
	BuffCheck_Display("Defaults were set to the following values: ")
	BuffCheck_ViewSettings()
	
end

-- SLASH COMMANDS --
BuffCheck_SlashCmdHandler = function(msg)
	if msg == "check" then
		BuffCheck_ClearTables()
	    BuffCheck_CheckBuffs()
	elseif msg == "announce" then
		BuffCheck_SavedVars.AnnounceMissing = true
		BuffCheck_btnAnnounce:SetChecked(true) -- Set checkbutton state
		BuffCheck_Display("Missing buffs will be displayed in raid chat.")
	elseif msg == "noannounce" then
		BuffCheck_SavedVars.AnnounceMissing = false
		BuffCheck_btnAnnounce:SetChecked(false) -- Set checkbutton state
		BuffCheck_Display("Missing buffs will be displayed locally.")
	elseif msg == "readycheck" then
		BuffCheck_SavedVars.CheckAtReadycheck = true
		BuffCheck_btnReadycheck:SetChecked(true) -- Set checkbutton state
		BuffCheck_Display("Buffs will be checked when a readycheck occurs.")
	elseif msg == "noreadycheck" then
		BuffCheck_SavedVars.CheckAtReadycheck = false
		BuffCheck_btnReadycheck:SetChecked(false) -- Set checkbutton state
		BuffCheck_Display("Buffs will NOT be checked when a readycheck occurs.")
	elseif msg == "reset" then
		BuffCheck_SetDefaults()
	elseif msg == "viewsettings" then
		BuffCheck_ViewSettings()
	elseif msg == "settings" or msg == "options" then
		InterfaceOptionsFrame_OpenToCategory(BuffCheck.panel.name)
		InterfaceOptionsFrame_OpenToCategory(BuffCheck.panel.name)
	elseif msg == "help" then
	    BuffCheck_Display("'/bc check' - Check buffs.")
		BuffCheck_Display("'/bc announce' - Display missing buffs in RAID CHAT.")
		BuffCheck_Display("'/bc noannounce' - Display missing buffs LOCALLY.")
		BuffCheck_Display("'/bc readycheck' - ENABLE display of missing buffs after a readycheck.")
		BuffCheck_Display("'/bc noreadycheck' - DISABLE display of missing buffs after a readycheck.")
		BuffCheck_Display("'/bc viewsettings' - View current settings.")
		BuffCheck_Display("'/bc settings' or '/bc options' - Open interface options.")
		BuffCheck_Display("'/bc reset' - Reset settings to defaults.")
	elseif msg == "test" then
	    -- Debug and test functions here --
		
    else
		BuffCheck_Display("Type '/bc help' for a command listing.")
	end
		
end -- SlashCmdHandler()

SLASH_BUFFCHECK1 = "/buffcheck"
SLASH_BUFFCHECK2 = "/bc"
SlashCmdList["BUFFCHECK"] = BuffCheck_SlashCmdHandler

-- EVENT HANDLERS AND SCRIPTS --===========================================================

-------------------------------------------------------------------------------
-- Sets global defaults if none are specified in the settings.
-- Calls CheckBuffs() on readycheck event if settings call for it.
-------------------------------------------------------------------------------
BuffCheck_EventFrame:SetScript("OnEvent", function(self, event, addon, ...)
	
	if event == "ADDON_LOADED" and addon == "BuffCheck" then
		
		if not BuffCheck_SavedVars.AnnounceMissing then
			-- Set saved settings to defaults
			BuffCheck_SavedVars.AnnounceMissing = BuffCheck_Defaults.AnnounceMissing
			BuffCheck_btnAnnounce:SetChecked(BuffCheck_Defaults.AnnounceMissing)
		end
		
		if not BuffCheck_SavedVars.CheckAtReadycheck then
			-- Set saved settings to defaults
			BuffCheck_SavedVars.CheckAtReadycheck = BuffCheck_Defaults.CheckAtReadycheck
			BuffCheck_btnReadycheck:SetChecked(BuffCheck_Defaults.CheckAtReadycheck)
		end
		
		-- Set state of checkbuttons within interface options
		BuffCheck_btnAnnounce:SetChecked(BuffCheck_SavedVars.AnnounceMissing)
		BuffCheck_btnReadycheck:SetChecked(BuffCheck_SavedVars.CheckAtReadycheck)
		
		-- DISPLAY LOAD MESSAGE --
		BuffCheck_Display("v"..VersionNum.." loaded.")
		BuffCheck_ViewSettings()
		
	end -- if event and addon
	
	if event == ("READY_CHECK_FINISHED") and (BuffCheck_SavedVars.CheckAtReadycheck) then
		BuffCheck_ClearTables()
		BuffCheck_CheckBuffs()
	end
	
end)

-------------------------------------------------------------------------------
-- Changes the settings table to match the state of the announce checkbutton.
-------------------------------------------------------------------------------
BuffCheck_btnAnnounce:SetScript("OnClick", function ()
	
	if BuffCheck_btnAnnounce:GetChecked() then
		BuffCheck_SavedVars.AnnounceMissing = true
	else
		BuffCheck_SavedVars.AnnounceMissing = false
	end
	
end)

-------------------------------------------------------------------------------
-- Changes the settings table to match the state of the readycheck checkbutton.
-------------------------------------------------------------------------------
BuffCheck_btnReadycheck:SetScript("OnClick", function ()
	
	if BuffCheck_btnReadycheck:GetChecked() then
		BuffCheck_SavedVars.CheckAtReadycheck = true
	else
		BuffCheck_SavedVars.CheckAtReadycheck = false
	end
	
end)

-------------------------------------------------------------------------------
-- Enables functionality of 'Defaults' button in the interface options.
-------------------------------------------------------------------------------
BuffCheck.panel.default = function() 
	
	BuffCheck_SetDefaults()
	
end
